@@ -3,7 +3,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020 Willem Vermin
+#-# 	      2019,2020,2021 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@ extern int    Santa_draw(cairo_t *cr);
 extern void   Santa_HandleCpuFactor(void);
 extern void   Santa_init(void);
 extern void   Santa_set_gc(void);
-extern int    Santa_ui(void);
+extern void   Santa_ui(void);
 
 extern float  ActualSantaSpeed;
 extern Region SantaPlowRegion;
