@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020 Willem Vermin
+#-# 	      2019,2020,2021 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include <math.h>
 #include <signal.h>
 #include <string.h>
+#include <assert.h>
 #include "Santa.h"
 #include "birds.h"
 #include "clocks.h"
@@ -105,127 +106,41 @@ static BirdType attrbird;
 
 
 
-int birds_ui()
+void birds_ui()
 {
-   int changes = 0;
+   UIDO(ShowBirds         ,                       );
+   UIDO(BirdsOnly         , ClearScreen();        );
+   UIDO(Neighbours        ,                       );
+   UIDO(Anarchy           ,                       );
+   UIDO(PrefDistance      ,                       );
+   UIDO(ViewingDistance   , attrbird2surface();   );
+   UIDO(BirdsSpeed        , birds_set_speed();    );
+   UIDO(AttrFactor        ,                       );
+   UIDO(DisWeight         ,                       );
+   UIDO(FollowWeight      ,                       );
+   UIDO(BirdsScale        , birds_set_scale();    );
+   UIDO(ShowAttrPoint     ,                       );
+   UIDOS(BirdsColor       , 
+	 birds_init_color(); 
+	 ClearScreen(););
+   UIDO(Nbirds            ,
+	 int start = OldFlags.Nbirds;
+	 if (Flags.Nbirds <= 0)
+	 Flags.Nbirds = 1;
+	 if (Flags.Nbirds > NBIRDS_MAX)
+	 Flags.Nbirds = NBIRDS_MAX;
+	 init_birds(start););
+   UIDO(FollowSanta, 
+	 if (!Flags.FollowSanta)
+	 birds_set_attraction_point_relative(0.5, 0.5, 0.5););
 
-   if(Flags.ShowBirds != OldFlags.ShowBirds)
-   {
-      OldFlags.ShowBirds = Flags.ShowBirds;
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.BirdsOnly != OldFlags.BirdsOnly)
-   {
-      P("BirdsOnly %d %d\n",Flags.BirdsOnly,OldFlags.BirdsOnly);
-      OldFlags.BirdsOnly = Flags.BirdsOnly;
-      ClearScreen();
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.Neighbours != OldFlags.Neighbours)
-   {
-      OldFlags.Neighbours = Flags.Neighbours;
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.Anarchy != OldFlags.Anarchy)
-   {
-      OldFlags.Anarchy = Flags.Anarchy;
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.PrefDistance != OldFlags.PrefDistance)
-   {
-      OldFlags.PrefDistance = Flags.PrefDistance;
-      changes++;
-      P("changes: %d\n",changes);
-   }
    if(Flags.BirdsRestart)
    {
       Flags.BirdsRestart = 0;
       init_birds(0);
       birds_set_attraction_point_relative(0.5, 0.5, 0.5);
-      P("changes: %d\n",changes);
+      P("Changes: %d\n",Flags.Changes);
    }
-   if(Flags.ViewingDistance != OldFlags.ViewingDistance)
-   {
-      OldFlags.ViewingDistance = Flags.ViewingDistance;
-      attrbird2surface();
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.BirdsSpeed != OldFlags.BirdsSpeed)
-   {
-      OldFlags.BirdsSpeed = Flags.BirdsSpeed;
-      birds_set_speed();
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.AttrFactor != OldFlags.AttrFactor)
-   {
-      OldFlags.AttrFactor = Flags.AttrFactor;
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.DisWeight != OldFlags.DisWeight)
-   {
-      OldFlags.DisWeight = Flags.DisWeight;
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.FollowWeight != OldFlags.FollowWeight)
-   {
-      OldFlags.FollowWeight = Flags.FollowWeight;
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.BirdsScale != OldFlags.BirdsScale)
-   {
-      OldFlags.BirdsScale = Flags.BirdsScale;
-      birds_set_scale();
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.ShowAttrPoint != OldFlags.ShowAttrPoint)
-   {
-      OldFlags.ShowAttrPoint = Flags.ShowAttrPoint;
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(strcmp(Flags.BirdsColor, OldFlags.BirdsColor))
-   {
-      P("%s %s\n",Flags.BirdsColor,OldFlags.BirdsColor);
-      birds_init_color();
-      ClearScreen();
-      free(OldFlags.BirdsColor);
-      OldFlags.BirdsColor = strdup(Flags.BirdsColor);
-      changes++;
-      P("changes: %d\n",changes);
-   }
-
-   if(Flags.Nbirds != OldFlags.Nbirds)
-   {
-      int start = OldFlags.Nbirds;
-      if (Flags.Nbirds <= 0)
-	 Flags.Nbirds = 1;
-      if (Flags.Nbirds > NBIRDS_MAX)
-	 Flags.Nbirds = NBIRDS_MAX;
-      OldFlags.Nbirds = Flags.Nbirds;
-      changes++;
-      P("changes: %d\n",changes);
-      init_birds(start);
-   }
-
-   if(Flags.FollowSanta != OldFlags.FollowSanta)
-   {
-      P("FollowSanta: %d->%d\n",OldFlags.FollowSanta,Flags.FollowSanta);
-      OldFlags.FollowSanta = Flags.FollowSanta;
-      if (!Flags.FollowSanta)
-	 birds_set_attraction_point_relative(0.5, 0.5, 0.5);
-      changes ++;
-   }
-   return changes;
 }
 
 static void normalize_speed(BirdType *bird, float speed)
@@ -524,7 +439,8 @@ int birds_draw(cairo_t *cr)
       if(before && Flags.FollowSanta && !Flags.BirdsOnly)
       {
 	 static int prevSantasize = -1;
-	 Santa_draw(cr);
+	 if (switches.UseGtk)
+	    Santa_draw(cr);
 	 attrbird.ix = SantaX+SantaWidth/2;
 	 attrbird.iz = SantaY+SantaHeight/2;
 	 switch(Flags.SantaSize)
@@ -571,7 +487,7 @@ int birds_draw(cairo_t *cr)
 	       GdkPixbuf       *pixbuf = 0;
 	       const GdkInterpType interpolation = GDK_INTERP_HYPER; 
 	       pixbuf = gdk_pixbuf_scale_simple(bird_pixbuf,iw,ih,interpolation); 
-	       cairo_surface_t *surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, 0, gdkwindow);
+	       cairo_surface_t *surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, 0, NULL);
 	       r2i(&testbird);
 	       cairo_set_source_surface (cr, surface, testbird.ix +(i-centerbird)*(iw+20), testbird.iz);
 	       my_cairo_paint_with_alpha(cr,ALPHA);
@@ -673,7 +589,7 @@ int birds_draw(cairo_t *cr)
 	       cache += iw*ih;
 	       P("Entries: %d Cache: %.0f MB width: %d Wing: %d orient: %d\n",table_counter,cache*4.0e-6,iw,nw,orient/8);
 	       GdkPixbuf *pixbuf = gdk_pixbuf_scale_simple(bird_pixbuf,iw,ih,interpolation); 
-	       table_insert(key,gdk_cairo_surface_create_from_pixbuf (pixbuf, 0, gdkwindow));
+	       table_insert(key,gdk_cairo_surface_create_from_pixbuf (pixbuf, 0, NULL));
 	       g_clear_object(&pixbuf);
 	    }
 	    surface = (cairo_surface_t*) table_get(key);
@@ -699,7 +615,8 @@ void init_birds(int start)
 {
    int i;
    P("nbirds: %d %d\n",start,Flags.Nbirds);
-   birds = (BirdType *)realloc(birds,sizeof(BirdType)*Flags.Nbirds);
+   // Bbirds+1 to prevent allocating zero bytes:
+   birds = (BirdType *)realloc(birds,sizeof(BirdType)*(Flags.Nbirds+1));
    if (kd)
       kd_free(kd);
    kd = kd_create(3);
@@ -780,11 +697,11 @@ void clear_flags()
 #define DOITB(what,type) \
    globals.what ## _changed = 0;
    DOITALLB();
-#undef DOITB
+#include "undefall.inc"
 #define DOITB(what) \
    globals.what ## _requested   = 0;
    BUTTONALL();
-#undef DOITB
+#include "undefall.inc"
 }
 
 void birds_set_speed()
