@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020 Willem Vermin
+#-# 	      2019,2020,2021 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 #pragma once
 extern void   blowoff_init(void);
 extern void   blowoff_draw(cairo_t *cr);
-extern int    blowoff_ui(void);
+extern void   blowoff_ui(void);
 extern int    BlowOff(void);
 extern void   InitBlowOffFactor(void);
 extern int    do_blowoff(gpointer data);
