@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020 Willem Vermin
+#-# 	      2019,2020,2021 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -55,6 +55,8 @@ static int      screen = 0;
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
+
+#include "vroot.h"
 /*
  * Other_stuff.h: Definitions of routines in other_stuff.
  *
