@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020 Willem Vermin
+#-# 	      2019,2020,2021 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -18,6 +18,9 @@
 #-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #-# 
 */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <stdio.h>
 #include <gtk/gtk.h>
 #include <stdlib.h>
@@ -83,43 +86,16 @@ void fallensnow_draw(cairo_t *cr)
    }
 }
 
-int fallensnow_ui()
+void fallensnow_ui()
 {
-   int changes = 0;
-   if(Flags.MaxWinSnowDepth != OldFlags.MaxWinSnowDepth)
-   {
-      OldFlags.MaxWinSnowDepth = Flags.MaxWinSnowDepth;
-      InitFallenSnow();
-      ClearScreen();
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.MaxScrSnowDepth != OldFlags.MaxScrSnowDepth)
-   {
-      OldFlags.MaxScrSnowDepth = Flags.MaxScrSnowDepth;
+   UIDO(MaxWinSnowDepth   , InitFallenSnow(); ClearScreen(); );
+   UIDO(MaxScrSnowDepth   , 
       SetMaxScreenSnowDepth();
       InitFallenSnow();
       ClearScreen();
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.NoKeepSBot != OldFlags.NoKeepSBot)
-   {
-      OldFlags.NoKeepSBot = Flags.NoKeepSBot;
-      InitFallenSnow();
-      ClearScreen();
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.NoKeepSWin != OldFlags.NoKeepSWin)
-   {
-      OldFlags.NoKeepSWin = Flags.NoKeepSWin;
-      InitFallenSnow();
-      ClearScreen();
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   return changes;
+      );
+   UIDO(NoKeepSBot        , InitFallenSnow(); ClearScreen(); );
+   UIDO(NoKeepSWin        , InitFallenSnow(); ClearScreen(); );
 }
 
 int do_fallen(UNUSED gpointer data)
@@ -312,7 +288,8 @@ Pixmap CreatePixmapFromFallen(FallenSnow *f)
    // todo: takes too much cpu
    int j;
    int p = 0;
-   unsigned char *bitmap = (unsigned char *) alloca((f->w8*f->h/8)*sizeof(unsigned char));
+   // malloc((1+ ...) to be sure we are allocating at least 1 byte
+   unsigned char *bitmap = (unsigned char *) malloc((1+f->w8*f->h/8)*sizeof(unsigned char));
 
    for (j=0; j<f->h; j++)
    {
@@ -334,6 +311,7 @@ Pixmap CreatePixmapFromFallen(FallenSnow *f)
       }
    }
    Pixmap pixmap = XCreateBitmapFromData(display, SnowWin, (char *)bitmap, f->w, f->h);
+   free(bitmap);
    return pixmap;
 }
 
@@ -424,7 +402,7 @@ void DrawFallen(FallenSnow *fsnow)
 void GenerateFlakesFromFallen(FallenSnow *fsnow, int x, int w, float vy)
 {
    P("GenerateFlakes %d %d %d %f\n",counter++,x,w,vy);
-   if (Flags.NoBlowSnow || Flags.NoSnowFlakes)
+   if (!Flags.BlowSnow || Flags.NoSnowFlakes)
       return;
    // animation of fallen fallen snow
    // x-values x..x+w are transformed in flakes, vertical speed vy
@@ -445,11 +423,13 @@ void GenerateFlakesFromFallen(FallenSnow *fsnow, int x, int w, float vy)
 	 for(k=0; k<kmax; k++)
 	 {
 	    float p = 0;
-	    if (!switches.UseGtk)
-	       p = drand48();
+	    //if (!switches.UseGtk)
+	    p = drand48();
 	    // In X11, (switches.UseGtk!=1) we want not too much
 	    // generated flakes
 	    // Otherwize, we go for more dramatic effects
+	    // But, it appeared that, if switches.UseGtk==1, too much snow
+	    // is generated, choking the x server. 
 	    if (p < 0.15)
 	    {
 	       Snow *flake   = MakeFlake(-1);
@@ -463,7 +443,9 @@ void GenerateFlakesFromFallen(FallenSnow *fsnow, int x, int w, float vy)
 		  flake->vx      = NewWind/8;
 	       flake->vy         = vy;
 	       flake->cyclic     = 0;
-	       if (switches.UseGtk && drand48() > 0.25)
+	       //if (switches.UseGtk && drand48() > 0.25)
+	       //if(drand48() > 0.25)
+	       if(0)  // next {} is not needed, it seems now
 	       {
 		  fluffify(flake,0.7);
 		  //flake->ry += 2*MaxSnowFlakeHeight*drand48();
