@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020 Willem Vermin
+#-# 	      2019,2020,2021 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -50,7 +50,7 @@ extern int    HandleFallenSnow(FallenSnow *fsnow);
 
 extern void   fallensnow_init(void);
 extern void   fallensnow_draw(cairo_t *cr);
-extern int    fallensnow_ui(void);
+extern void   fallensnow_ui(void);
 extern void   CleanFallenArea(FallenSnow *fsnow, int x, int w);
 extern void   CleanFallen(Window id);
 extern void   DrawFallen(FallenSnow *fsnow);
