@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020 Willem Vermin
+#-# 	      2019,2020,2021 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -22,12 +22,30 @@
 #include <X11/Xlib.h>
 #include "doit.h"
 
-#define DOIT_I(x) int x; 
-#define DOIT_L(x) unsigned long int x;
-#define DOIT_S(x) char *x;
+#define UIDO(_x,_y) \
+   if(Flags._x != OldFlags._x) \
+{ \
+   if(Flags.Noisy) { printf("%-16s %6d: %-22s %8d -> %8d\n",__FILE__,__LINE__,#_x,OldFlags._x, Flags._x); fflush(NULL); } \
+   {_y} \
+   OldFlags._x = Flags._x; \
+   Flags.Changes++; \
+}
 
-typedef struct flags {
-   //unsigned long int WindowId;
+#define UIDOS(_x,_y) \
+   if(strcmp(Flags._x, OldFlags._x)) \
+{ \
+   if(Flags.Noisy) { printf("%-16s %6d: %-22s %8s -> %8s\n",__FILE__,__LINE__,#_x,OldFlags._x, Flags._x); fflush(NULL); } \
+   {_y} \
+   free(OldFlags._x); \
+   OldFlags._x = strdup(Flags._x); \
+   Flags.Changes++; \
+}
+
+#define DOIT_I(x,d,v) int x;
+#define DOIT_L(x,d,v) unsigned long int x;
+#define DOIT_S(x,d,v) char *x;
+
+typedef struct _flags {
    DOITALL
       int dummy;
 }FLAGS;
@@ -37,8 +55,9 @@ typedef struct flags {
 
 extern FLAGS Flags;
 extern FLAGS OldFlags;
+extern FLAGS DefaultFlags;
+extern FLAGS VintageFlags;
 
 extern int  HandleFlags(int argc, char*argv[]);
 extern void InitFlags(void);
-extern void PrintVersion(void);
 extern void WriteFlags(void);
