@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020 Willem Vermin
+#-# 	      2019,2020,2021 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -45,9 +45,6 @@
 #include <config.h>
 #endif
 #include <X11/Intrinsic.h>
-#ifdef HAVE_ALLOCA_H
-#include <alloca.h>
-#endif
 #include <ctype.h>
 #include <gtk/gtk.h>
 #include <stdlib.h>
@@ -64,6 +61,7 @@
 #include "flags.h"
 #include "mainstub.h"
 #include "meteo.h"
+#include "moon.h"
 #include "Santa.h"
 #include "scenery.h"
 #include "snow.h"
@@ -82,16 +80,14 @@
 #include "loadmeasure.h"
 #include "varia.h"
 
+#include "vroot.h"
+
 #ifdef DEBUG
 #undef DEBUG
 #endif
 //#define DEBUG
 
 
-// from flags.h
-FLAGS Flags;
-FLAGS OldFlags;
-
 // from windows.h
 Display *display;
 int     screen;
@@ -110,14 +106,13 @@ double  cpufactor = 1.0;
 float   NewWind = 0;
 
 GtkWidget       *drawing_area = NULL;
-GdkWindow       *gdkwindow = NULL;
-
-
 
 // miscellaneous
 char       Copyright[] = "\nXsnow\nCopyright 1984,1988,1990,1993-1995,2000-2001 by Rick Jansen, all rights reserved, 2019,2020 also by Willem Vermin\n";
 static int HaltedByInterrupt = 0;
 
+static char **Argv;
+static int Argc;
 
 // timing stuff
 //static double       TotSleepTime = 0;
@@ -145,6 +140,8 @@ static void   drawit(cairo_t *cr);
 static void   restart_do_draw_all(void);
 static int    myDetermineWindow(void);
 static void   set_below_above(void);
+static void   change_ww(void);
+static void   DoAllWorkspaces(void);
 
 
 static void Thanks(void)
@@ -281,16 +278,18 @@ int main_c(int argc, char *argv[])
    signal(SIGTERM, SigHandler);
    signal(SIGHUP,  SigHandler);
    srand48((long int)(wallcl()*1.0e6));
+
    int i;
    // make a copy of all flags, before gtk_init() maybe removes some.
    // we need this at a restart of the program.
 
-   char **Argv;
    Argv = (char**) malloc((argc+1)*sizeof(char**));
    for (i=0; i<argc; i++)
       Argv[i] = strdup(argv[i]);
    Argv[argc] = NULL;
+   Argc = argc;
 
+   InitFlags();
    // we search for flags that only produce output to stdout,
    // to enable to run in a non-X environment, in which case 
    // gtk_init() would fail.
@@ -313,6 +312,7 @@ int main_c(int argc, char *argv[])
 	 return 0;
       }
    }
+   printf("Xsnow running in GTK version: %s\n",ui_gtk_version());
 
    // Circumvent wayland problems:before starting gtk: make sure that the 
    // gdk-x11 backend is used.
@@ -327,7 +327,6 @@ int main_c(int argc, char *argv[])
       IsWayland = 0;
 
    gtk_init(&argc, &argv);
-   InitFlags();
    int rc = HandleFlags(argc, argv);
    switch(rc)
    {
@@ -347,8 +346,25 @@ int main_c(int argc, char *argv[])
    if (!Flags.NoConfig)
       WriteFlags();
 
+   if(Flags.CheckGtk && !ui_checkgtk() && !Flags.NoMenu)
+   {
+      printf("Xsnow needs gtk version >= %s, found version %s\n",ui_gtk_required(),ui_gtk_version());
+
+      if(!ui_run_nomenu())
+      {
+	 Thanks();
+	 return 0;
+      }
+      else
+      {
+	 Flags.NoMenu = 1;
+	 printf("Continuing with flag '-nomenu'\n");
+      }
+   }
+
    display = XOpenDisplay(Flags.DisplayName);
-   RootWindow = DefaultRootWindow(display);
+   if(0)  // to prevent warning about unused function in vroot.h
+      Rootwindow = DefaultRootWindow(display);  // Rootwindow will be set in DetermineWindow() in windows.c 
    XSynchronize(display,dosync);
    XSetErrorHandler(XsnowErrors);
    screen = DefaultScreen(display);
@@ -391,7 +407,7 @@ int main_c(int argc, char *argv[])
    //   - clearing is done writing the same image, but with color black (0x00000000) 
    //   else
    //   - we will use XClearArea to erase flakes and the like. This works well
-   //     on fvwm-like desktops (desktop == RootWindow) with exposures set to 0
+   //     on fvwm-like desktops (desktop == Rootwindow) with exposures set to 0
    //     It works more or less in for example KDE, but exposures must be set to 1
    //     which severely stresses plasma shell (or nautilus-desktop in Gnome, 
    //     but we do not use XClearArea in Gnome).
@@ -412,6 +428,7 @@ int main_c(int argc, char *argv[])
    //                                          unpleasant things happen when a snowflake
    //                                          is in the trajectory of a meteorite
    windows_init();
+   moon_init();
    Santa_init();
    birds_init();
    scenery_init();
@@ -436,13 +453,11 @@ int main_c(int argc, char *argv[])
 
    HandleCpuFactor();
 
-#define DOIT_I(x) OldFlags.x = Flags.x;
-#define DOIT_L(x) DOIT_I(x);
-#define DOIT_S(x) OldFlags.x = strdup(Flags.x);
+#define DOIT_I(x,d,v) OldFlags.x = Flags.x;
+#define DOIT_L(x,d,v) DOIT_I(x,d,v);
+#define DOIT_S(x,d,v) OldFlags.x = strdup(Flags.x);
    DOITALL;
-#undef DOIT_I
-#undef DOIT_L
-#undef DOIT_S
+#include "undefall.inc"
 
 
    OldFlags.FullScreen = !Flags.FullScreen;
@@ -472,6 +487,7 @@ int main_c(int argc, char *argv[])
 	 ui_gray_below(1);
 	 ui_gray_birds(1);
 	 ui_set_birds_header("No alpha channel: no birds will fly.");
+	 ui_set_celestials_header("No alpha channel: no moon.");
       }
       ui_set_sticky(Flags.AllWorkspaces);
       add_to_mainloop(PRIORITY_DEFAULT, 2.0, do_show_desktop_type, NULL);
@@ -491,6 +507,7 @@ int main_c(int argc, char *argv[])
    {
       sleep(0);
       printf("Xsnow restarting: %s\n",Argv[0]);
+      fflush(NULL);
       execvp(Argv[0],Argv);
    }
    else
@@ -498,24 +515,34 @@ int main_c(int argc, char *argv[])
    return 0;
 }		/* End of snowing */
 
+
+
 void set_below_above()
 {
    P("%d set_below_above\n",counter++);
+   XWindowChanges changes;
+   // to be sure: we do it in gtk mode and window mode
    if (Flags.BelowAll)
    {
       setbelow(GTK_WINDOW(TransA));
       setbelow(GTK_WINDOW(TransB));
+      changes.stack_mode = Below;
+      if(SnowWina)XConfigureWindow(display,SnowWina,CWStackMode,&changes);
+      if(SnowWinb)XConfigureWindow(display,SnowWinb,CWStackMode,&changes);
    }
    else
    {
       setabove(GTK_WINDOW(TransA));
       setabove(GTK_WINDOW(TransB));
+      changes.stack_mode = Above;
+      if(SnowWina)XConfigureWindow(display,SnowWina,CWStackMode,&changes);
+      if(SnowWinb)XConfigureWindow(display,SnowWinb,CWStackMode,&changes);
    }
 }
 
 int myDetermineWindow()
 {
-   P("myDetermineWindow root: %#lx\n",RootWindow);
+   P("myDetermineWindow root: %#lx\n",Rootwindow);
 
    if (drawconnect)
    {
@@ -526,7 +553,32 @@ int myDetermineWindow()
 
    static char * SnowWinaName = NULL;
    static char * SnowWinbName = NULL;
-   static int IsDesktop;
+   static int IsDesktop = 0;
+
+   if(IsDesktop)
+      if(1)
+      {
+	 if (SnowWina)
+	 {
+	    XDestroyWindow(display,SnowWina);
+	    SnowWina = 0;
+	 }
+	 if (SnowWinb)
+	 {
+	    XDestroyWindow(display,SnowWinb);
+	    SnowWinb = 0;
+	 }
+	 if (TransA)
+	 {
+	    gtk_widget_destroy(TransA);
+	    TransA = NULL;
+	 }
+	 if (TransB)
+	 {
+	    gtk_widget_destroy(TransB);
+	    TransB = NULL;
+	 }
+      }
    if (!SnowWina)
    {
       if (SnowWinaName)
@@ -593,15 +645,15 @@ int myDetermineWindow()
 
 	 switches.UseGtk    = 1;
 	 switches.DrawBirds = 1;
-	 switches.Trans     = 0;
+	 switches.Trans     = 1;  // should be 1? todo
 	 switches.Root      = 0;
 	 switches.Desktop   = 1;
       }
    }
    else                          //  No transparent window: Scenario 4
    {
-      P("Scenario 4 Desktop: %d\n",IsDesktop);
-      printf("Scenario: Use X11 for drawing snow in root window, no birds will fly.\n");
+      P("Scenario 4 IsDesktop: %d\n",IsDesktop);
+      printf("Scenario: Use X11 for drawing snow, no birds will fly, no moon will shine.\n");
       // in LXDE, SnowWin will be overwritten by id of window pcmanfm
       SnowWin            = SnowWina;
       SnowWinName        = SnowWinaName;
@@ -612,16 +664,10 @@ int myDetermineWindow()
       switches.Desktop   = IsDesktop;
    }
 
-   InitDisplayDimensions();
-
-   P("windows: SnowWin:%s SnowWina:%s SnowWinb:%s\n",SnowWinName,SnowWinaName,SnowWinbName);
-   printf("Snowing in window: %#lx - \"%s\" - depth: %d - geom: %d %d %dx%d - alpha: %s - exposures: %d\n",
-	 SnowWin,SnowWinName,SnowWinDepth,
-	 SnowWinX,SnowWinY,SnowWinWidth,SnowWinHeight, TransA?"yes":"no",switches.Exposures);
    if (TransA)
       printf("Birds in window: %#lx - \"%s\"\n",SnowWina,SnowWinaName);
    else
-      printf("No birds (you need a compositing display manager to let birds fly).\n");
+      printf("No birds, no moon (you need a compositing display manager to let birds fly and monn shine).\n");
 
    if(TransA)
    {
@@ -636,10 +682,85 @@ int myDetermineWindow()
    {
       drawconnect = 0;
    }
+
+   InitDisplayDimensions();
+
+   static int first_call = 1;
+   static int xa = 0;
+   static int ya = 0;
+
+   if (first_call)
+   {
+      first_call = 0;
+      xa = SnowWinX;
+      ya = SnowWinY;
+   }
+
+
+   if(switches.Desktop)
+   {
+      if(Flags.MoveWindow)
+      {
+	 P("XM: %d %d\n",0,0);
+	 if(SnowWina)XMoveWindow(display,SnowWina,0,0);
+	 if(SnowWinb)XMoveWindow(display,SnowWinb,0,0);
+      }
+      else
+      {
+	 P("XM: %d %d\n",xa,ya);
+	 if(SnowWina)XMoveWindow(display,SnowWina,xa,ya);
+	 if(SnowWinb)XMoveWindow(display,SnowWinb,xa,ya);
+      }
+   }
+
+   P("windows: SnowWin:%s SnowWina:%s SnowWinb:%s\n",SnowWinName,SnowWinaName,SnowWinbName);
+   printf("Snowing in window: %#lx - \"%s\" - depth: %d - geom: %d %d %dx%d - alpha: %s - exposures: %d\n",
+	 SnowWin,SnowWinName,SnowWinDepth,
+	 SnowWinX,SnowWinY,SnowWinWidth,SnowWinHeight, TransA?"yes":"no",switches.Exposures);
+   fflush(NULL);
    return 1;
 }
 
 
+void change_ww()
+{
+   ClearScreen();
+   P("WantWindow: %d\n",Flags.WantWindow);
+   if(draw_all_id)
+   {
+      g_source_remove(draw_all_id);
+      P("removed %d\n",draw_all_id);
+   }
+   draw_all_id = 0;
+   myDetermineWindow();
+   ui_gray_erase(switches.UseGtk);
+   P("WantWindow: %d\n",Flags.WantWindow);
+}
+
+
+void DoAllWorkspaces()
+{
+   if(Flags.AllWorkspaces)
+   {
+      P("stick\n");
+      if (switches.UseGtk||switches.Trans)
+      {
+	 gtk_window_stick(GTK_WINDOW(TransA));
+	 gtk_window_stick(GTK_WINDOW(TransB));
+      }
+   }
+   else
+   {
+      P("unstick\n");
+      if (switches.UseGtk||switches.Trans)
+      {
+	 gtk_window_unstick(GTK_WINDOW(TransA));
+	 gtk_window_unstick(GTK_WINDOW(TransB));
+      }
+   }
+   ui_set_sticky(Flags.AllWorkspaces);
+}
+
 // here we are handling the buttons in ui
 // Ok, this is a long list, and could be implemented more efficient.
 // But, do_ui_check is called not too frequently, so....
@@ -653,158 +774,49 @@ int do_ui_check(UNUSED gpointer data)
    if (Flags.NoMenu)
       return TRUE;
 
-   int changes = 0;
-   changes += Santa_ui();
-   changes += scenery_ui();
-   changes += birds_ui();
-   changes += snow_ui();
-   changes += meteo_ui();
-   changes += wind_ui();
-   changes += stars_ui();
-   changes += fallensnow_ui();
-   changes += blowoff_ui();
-   changes += treesnow_ui();
+   Santa_ui();
+   scenery_ui();
+   birds_ui();
+   snow_ui();
+   meteo_ui();
+   wind_ui();
+   stars_ui();
+   fallensnow_ui();
+   blowoff_ui();
+   treesnow_ui();
+   moon_ui();
+   ui_ui();
 
-   if (Flags.WantWindow != OldFlags.WantWindow)
-   {
-      ClearScreen();
-      OldFlags.WantWindow = Flags.WantWindow;
-      P("WantWindow: %d\n",Flags.WantWindow);
-      if(draw_all_id)
-      {
-	 g_source_remove(draw_all_id);
-	 P("removed %d\n",draw_all_id);
-      }
-      draw_all_id = 0;
-      myDetermineWindow();
-      ui_gray_erase(switches.UseGtk);
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.CpuLoad != OldFlags.CpuLoad)
-   {
-      OldFlags.CpuLoad = Flags.CpuLoad;
-      P("cpuload: %d %d\n",OldFlags.CpuLoad,Flags.CpuLoad);
-      HandleCpuFactor();
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.Transparency != OldFlags.Transparency)
-   {
-      OldFlags.Transparency = Flags.Transparency;
-      P("Transparency: %d %d\n",OldFlags.Transparency,Flags.Transparency);
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.UseBG != OldFlags.UseBG)
-   {
-      OldFlags.UseBG = Flags.UseBG;
-      SetGCFunctions();
-      ClearScreen();
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(strcmp(Flags.BGColor,OldFlags.BGColor))
-   {
-      free(OldFlags.BGColor);
-      OldFlags.BGColor = strdup(Flags.BGColor);
-      if(Flags.UseBG)
-      {
+   UIDO (WantWindow          , change_ww();                     );
+   UIDO (CpuLoad             , HandleCpuFactor();               );
+   UIDO (Transparency        ,                                  );
+   UIDO (UseBG               , SetGCFunctions(); ClearScreen(); );
+   UIDOS(BGColor             , 
+	 if(Flags.UseBG)
+	 {
 	 SetGCFunctions();
 	 ClearScreen();
-      }
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.Exposures != OldFlags.Exposures)
-   {
-      P("changes: %d %d %d\n",changes,OldFlags.Exposures,Flags.Exposures);
-      OldFlags.Exposures = Flags.Exposures;
-      HandleExposures();
-      HandleCpuFactor();
-      ClearScreen();
-      changes++;
-      P("changes: %d %d %d\n",changes,OldFlags.Exposures,Flags.Exposures);
-      P("changes: %d\n",changes);
-   }
-   if(Flags.OffsetS != OldFlags.OffsetS)
-   {
-      OldFlags.OffsetS = Flags.OffsetS;
-      changes++;
-      P("changes: %d %d\n",changes,Flags.OffsetS);
-   }
-   if(Flags.OffsetY != OldFlags.OffsetY)
-   {
-      OldFlags.OffsetY = Flags.OffsetY;
-      changes++;
-      P("changes: %d %d\n",changes,Flags.OffsetY);
-   }
-   if(Flags.NoFluffy != OldFlags.NoFluffy)
-   {
-      OldFlags.NoFluffy = Flags.NoFluffy;
-      ClearScreen();
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.FullScreen != OldFlags.FullScreen)
-   {
-      OldFlags.FullScreen = Flags.FullScreen;
-      if(TransA)
-      {
-	 if (Flags.FullScreen)
-	 {
-	    gtk_window_fullscreen(GTK_WINDOW(TransA));
-	    gtk_window_fullscreen(GTK_WINDOW(TransB));
 	 }
-	 else
-	 {
-	    gtk_window_unfullscreen(GTK_WINDOW(TransA));
-	    gtk_window_unfullscreen(GTK_WINDOW(TransB));
-	 }
-	 set_below_above();
-      }
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.AllWorkspaces != OldFlags.AllWorkspaces)
-   {
-      if(Flags.AllWorkspaces)
-      {
-	 P("stick\n");
-	 if (switches.UseGtk||switches.Trans)
-	 {
-	    gtk_window_stick(GTK_WINDOW(TransA));
-	    gtk_window_stick(GTK_WINDOW(TransB));
-	 }
-      }
-      else
-      {
-	 P("unstick\n");
-	 if (switches.UseGtk||switches.Trans)
-	 {
-	    gtk_window_unstick(GTK_WINDOW(TransA));
-	    gtk_window_unstick(GTK_WINDOW(TransB));
-	 }
-      }
-      OldFlags.AllWorkspaces = Flags.AllWorkspaces;
-      ui_set_sticky(Flags.AllWorkspaces);
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.BelowAll != OldFlags.BelowAll)
-   {
-      OldFlags.BelowAll = Flags.BelowAll;
-      set_below_above();
-      changes++;
-      P("changes: %d %d\n",changes,Flags.BelowAll);
-   }
+	);
+   UIDO (Exposures           ,
+	 HandleExposures();
+	 HandleCpuFactor();
+	 ClearScreen();
+	);
+   UIDO (OffsetS             ,                                  );
+   UIDO (OffsetY             ,                                  );
+   UIDO (NoFluffy            , ClearScreen();                   );
+   UIDO (MoveWindow          , myDetermineWindow();             );
+   UIDO (AllWorkspaces       , DoAllWorkspaces();               );
+   UIDO (BelowAll            , set_below_above();               );
 
-   if (changes > 0)
+   if (Flags.Changes > 0)
    {
       P("WriteFlags\n");
       WriteFlags();
-      P("-----------changes: %d\n",changes);
+      P("-----------Changes: %d\n",Flags.Changes);
    }
+   Flags.Changes = 0;
    return TRUE;
 }
 
@@ -935,6 +947,8 @@ int do_show_desktop_type(UNUSED gpointer data)
 int do_testing(UNUSED gpointer data)
 {
    counter++;
+   //Flags.ThemeXsnow = 1-Flags.ThemeXsnow;
+   P("testing ThemeXsnow:%d\n",Flags.ThemeXsnow);
    if (Flags.Done)
       return FALSE;
    return TRUE;
@@ -1011,15 +1025,22 @@ void drawit(cairo_t *cr)
    if (Flags.Done)
       return;
 
+
+   int skipit = !switches.UseGtk || Flags.BirdsOnly || !WorkspaceActive();
+
+   if (!skipit)
+   {
+      stars_draw(cr);
+      meteo_draw(cr);
+   }
+
+   moon_draw(cr);
+
    birds_draw(cr);
 
-   if (!switches.UseGtk || Flags.BirdsOnly || !WorkspaceActive())
+   if (skipit)
       return;
 
-   stars_draw(cr);
-
-   meteo_draw(cr);
-
    scenery_draw(cr);
 
    if(!Flags.FollowSanta || !Flags.ShowBirds) // if Flags.FollowSanta: drawing of Santa is done in birds_draw()
