@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020 Willem Vermin
+#-# 	      2019,2020,2021 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@ typedef struct _MeteoMap {
 
 
 extern void meteo_init(void);
-extern int  meteo_ui(void);
+extern void meteo_ui(void);
 extern void meteo_draw(cairo_t *cr);
 extern void meteo_erase(void);
 
