@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020 Willem Vermin
+#-# 	      2019,2020,2021 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -27,8 +27,11 @@
 
 //extern SnowMap       snowPix[];
 extern StarMap       starPix;
-extern const char    ***Santas[][2];
-extern const char    **xpmtrees[];
-extern const char    **xsnow_logo;
-extern const char    **birds_xpm[];
-extern const char    **snow_xpm[];
+extern XPM_TYPE    ***Santas[][2];
+extern XPM_TYPE    **xpmtrees[];
+extern XPM_TYPE    **xsnow_logo;
+extern XPM_TYPE    **birds_xpm[];
+extern XPM_TYPE    **snow_xpm[];
+extern XPM_TYPE    **moons_xpm[];
+
+extern int NMOONPIXBUFS;
