@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020 Willem Vermin
+#-# 	      2019,2020,2021 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -19,8 +19,14 @@
 #-# 
 */
 
+#define DEFAULTTREETYPE 2
+
 #define NOTACTIVE \
    (Flags.BirdsOnly || !WorkspaceActive())
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <stdio.h>
 #include <gtk/gtk.h>
 #include <stdlib.h>
@@ -89,52 +95,13 @@ int scenery_draw(cairo_t *cr)
    return TRUE;
 }
 
-int scenery_ui()
+void scenery_ui()
 {
-   int changes = 0;
-
-   if(strcmp(Flags.TreeType, OldFlags.TreeType))
-   {
-      P("Treetype %s %s\n",Flags.TreeType,OldFlags.TreeType);
-      RedrawTrees();
-      free(OldFlags.TreeType);
-      OldFlags.TreeType = strdup(Flags.TreeType);
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.DesiredNumberOfTrees != OldFlags.DesiredNumberOfTrees)
-   {
-      RedrawTrees();
-      OldFlags.DesiredNumberOfTrees = Flags.DesiredNumberOfTrees;
-      changes++;
-      P("NTREES: %d %d\n",OldFlags.DesiredNumberOfTrees,Flags.DesiredNumberOfTrees);
-      P("changes: %d\n",changes);
-   }
-   if(Flags.TreeFill != OldFlags.TreeFill)
-   {
-      RedrawTrees();
-      OldFlags.TreeFill = Flags.TreeFill;
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.NoTrees != OldFlags.NoTrees)
-   {
-      RedrawTrees();
-      OldFlags.NoTrees = Flags.NoTrees;
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(strcmp(Flags.TreeColor, OldFlags.TreeColor))
-   {
-      P("%s %s\n",Flags.TreeColor,OldFlags.TreeColor);
-      ReInitTree0();
-      free(OldFlags.TreeColor);
-      OldFlags.TreeColor = strdup(Flags.TreeColor);
-      changes++;
-      P("changes: %d\n",changes);
-   }
-
-   return changes;
+   UIDOS(TreeType               , RedrawTrees(););
+   UIDO (DesiredNumberOfTrees   , RedrawTrees(););
+   UIDO (TreeFill               , RedrawTrees(););
+   UIDO (NoTrees                , RedrawTrees(););
+   UIDOS(TreeColor              , ReInitTree0(););
 }
 
 void RedrawTrees()
@@ -172,7 +139,7 @@ void create_tree_surface(int tt,int flip, const char **xpm)
    }
    else
       pixbuf = pixbuf1;
-   tree_surfaces[tt][flip] = gdk_cairo_surface_create_from_pixbuf (pixbuf, 0, gdkwindow);
+   tree_surfaces[tt][flip] = gdk_cairo_surface_create_from_pixbuf (pixbuf, 0, NULL);
    g_clear_object(&pixbuf);
    sscanf(xpmtrees[tt][0],"%d %d",&TreeWidth[tt],&TreeHeight[tt]);
 
@@ -395,7 +362,7 @@ void ReInitTree0()
    int i;
    int n = TreeHeight[0]+3;
    //char *xpmtmp[n];
-   char **xpmtmp = (char **)alloca(n*sizeof(char *));
+   char **xpmtmp = (char **)malloc(n*sizeof(char *));
    int j;
    for (j=0; j<2; j++)
       xpmtmp[j] = strdup(xpmtrees[0][j]);
@@ -414,6 +381,7 @@ void ReInitTree0()
    }
    for (j=0; j<n; j++)
       free(xpmtmp[j]);
+   free(xpmtmp);
 }
 
 
