@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020 Willem Vermin
+#-# 	      2019,2020,2021 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include <gtk/gtk.h>
 #include <stdlib.h>
 #include <math.h>
+#include <assert.h>
 #include "debug.h"
 #include "xsnow.h"
 #include "pixmaps.h"
@@ -71,6 +72,7 @@ static void   DelFlake(Snow *flake);
 static void   DrawSnowFlake(Snow *flake);
 static void   genxpmflake(char ***xpm, int w, int h);
 static void   add_random_flakes(int n);
+static void   SetSnowSize(void);
 
 
 Region       NoSnowArea_dynamic;
@@ -78,7 +80,6 @@ Pixel        SnowcPix;
 unsigned int MaxSnowFlakeHeight = 0;  /* Highest flake */
 unsigned int MaxSnowFlakeWidth  = 0;  /* Widest  flake */
 int          FlakeCount         = 0;  /* # active flakes */
-int          UseVintageFlakes   = 0;  /* whether to use only vintage flakes */
 int          FluffCount         = 0;
 
 void snow_init()
@@ -149,64 +150,33 @@ void snow_set_gc()
    }
 }
 
-int snow_ui()
+void SetSnowSize()
 {
-   int changes = 0;
+   add_random_flakes(EXTRA_FLAKES);
+   init_snow_surfaces();
+   init_snow_pix();
+   snow_set_gc();
+   ClearScreen();
+   // the following, otherwize we see often double flakes
+   // why? race condition in x server?
+   if (!switches.UseGtk)
+      add_to_mainloop(PRIORITY_DEFAULT, 0.1, do_initsnow ,NULL);
+}
 
-   if(Flags.NoSnowFlakes != OldFlags.NoSnowFlakes)
-   {
-      OldFlags.NoSnowFlakes = Flags.NoSnowFlakes;
-      if(Flags.NoSnowFlakes)
-	 ClearScreen();
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.SnowFlakesFactor != OldFlags.SnowFlakesFactor)
-   {
-      OldFlags.SnowFlakesFactor = Flags.SnowFlakesFactor;
-      InitFlakesPerSecond();
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(strcmp(Flags.SnowColor, OldFlags.SnowColor))
-   {
-      InitSnowColor();
-      ClearScreen();
-      free(OldFlags.SnowColor);
-      OldFlags.SnowColor = strdup(Flags.SnowColor);
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.SnowSpeedFactor != OldFlags.SnowSpeedFactor)
-   {
-      OldFlags.SnowSpeedFactor = Flags.SnowSpeedFactor;
-      InitSnowSpeedFactor();
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.FlakeCountMax != OldFlags.FlakeCountMax)
-   {
-      OldFlags.FlakeCountMax = Flags.FlakeCountMax;
-      changes++;
-      P("changes: %d\n",changes);
-   }
-   if(Flags.SnowSize != OldFlags.SnowSize)
-   {
-      OldFlags.SnowSize = Flags.SnowSize;
-      add_random_flakes(EXTRA_FLAKES);
-      init_snow_surfaces();
-      init_snow_pix();
-      snow_set_gc();
-      ClearScreen();
-      // the following, otherwize we see often double flakes
-      // why? race condition in x server?
-      if (!switches.UseGtk)
-	 add_to_mainloop(PRIORITY_DEFAULT, 0.1, do_initsnow ,NULL);
-      changes++;
-      P("changes: %d %d\n",changes,Flags.SnowSize);
-   }
-
-   return changes;
+void snow_ui()
+{
+   UIDO (NoSnowFlakes, 
+	 if(Flags.NoSnowFlakes)
+	 ClearScreen();                                            );
+   UIDO (SnowFlakesFactor               , InitFlakesPerSecond();   );
+   UIDOS(SnowColor                      ,
+	 InitSnowColor();
+	 ClearScreen();                                            );
+   UIDO (SnowSpeedFactor                , InitSnowSpeedFactor();   );
+   UIDO (FlakeCountMax                  ,                          );
+   UIDO (SnowSize                       , 
+	 SetSnowSize(); 
+	 Flags.VintageFlakes = 0;                                  );
 }
 
 void init_snow_surfaces()
@@ -223,7 +193,7 @@ void init_snow_surfaces()
       xpm_destroy(x);
       if (snow_surfaces[i])
 	 cairo_surface_destroy(snow_surfaces[i]);
-      snow_surfaces[i] = gdk_cairo_surface_create_from_pixbuf (pixbuf, 0, gdkwindow);
+      snow_surfaces[i] = gdk_cairo_surface_create_from_pixbuf (pixbuf, 0, NULL);
       //snow_surfaces[i] = gdk_cairo_surface_create_from_pixbuf (pixbuf, 1, NULL);
       g_clear_object(&pixbuf);
       /*
@@ -280,6 +250,7 @@ int snow_draw(cairo_t *cr)
 {
    if (Flags.NoSnowFlakes)
       return TRUE;
+   P("snow_draw %d\n",counter++);
 
    set_begin();
    Snow *flake;
@@ -351,7 +322,7 @@ int do_UpdateSnowFlake(Snow *flake)
    if(NOTACTIVE)
       return TRUE;
    //P(" ");printflake(flake);
-   
+
    double FlakesDT = time_snowflakes;
    float NewX = flake->rx + (flake->vx*FlakesDT)*SnowSpeedFactor;
    float NewY = flake->ry + (flake->vy*FlakesDT)*SnowSpeedFactor;
@@ -582,7 +553,7 @@ int do_UpdateSnowFlake(Snow *flake)
 		  grec.height = rec.height;
 		  cairo_region_union_rectangle(gSnowOnTreesRegion,&grec);
 
-		  if(!Flags.NoBlowSnow && OnTrees < Flags.MaxOnTrees)
+		  if(Flags.BlowSnow && OnTrees < Flags.MaxOnTrees)
 		  {
 		     SnowOnTrees[OnTrees].x = rec.x;
 		     SnowOnTrees[OnTrees].y = rec.y;
@@ -651,7 +622,7 @@ Snow *MakeFlake(int type)
    FlakeCount++; 
    if (type < 0)
    {
-      if (UseVintageFlakes)
+      if (Flags.VintageFlakes)
 	 type = drand48()*NFlakeTypesVintage;
       else
 	 type = NFlakeTypesVintage + drand48()*(MaxFlakeTypes - NFlakeTypesVintage);
@@ -891,6 +862,7 @@ void genxpmflake(char ***xpm, int w, int h)
    if (nw == 1 && nh == 1)
       nh = 2;
 
+   assert(nh>0);
 
    P("allocating %d\n",(nh+3)*sizeof(char*));
    *xpm = (char **)malloc((nh+3)*sizeof(char*));
@@ -905,6 +877,7 @@ void genxpmflake(char ***xpm, int w, int h)
 
    int offset = 3;
    P("allocating %d\n",nw+1);
+   assert(nw>=0);
    for (i=0; i<nh; i++)
       X[i+offset] = (char *) malloc((nw+1)*sizeof(char));
 
@@ -921,6 +894,10 @@ void genxpmflake(char ***xpm, int w, int h)
       X[offset + (int)(ya[i]-ymin)] [(int)(xa[i]-xmin)] = c;
       P("%f %f\n",ya[i]-ymin,xa[i]-xmin);
    }
+   free(x);
+   free(y);
+   free(xa);
+   free(ya);
 }
 
 void add_random_flakes(int n)
