@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020 Willem Vermin
+#-# 	      2019,2020,2021 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -91,7 +91,8 @@ void init_stars()
    int i;
    NStars = Flags.NStars;
    P("initstars %d\n",NStars);
-   Stars = (Skoordinaten *) realloc(Stars,NStars*sizeof(Skoordinaten));
+   // Nstars+1: we do not allocate 0 bytes
+   Stars = (Skoordinaten *) realloc(Stars,(NStars+1)*sizeof(Skoordinaten));
    for (i=0; i<NStars; i++)
    {
       Skoordinaten *star = &Stars[i];
@@ -104,6 +105,8 @@ void init_stars()
 
 void stars_draw(cairo_t *cr)
 {
+   if (!Flags.Stars)
+      return;
    int i;
    cairo_save(cr);
    cairo_set_line_width(cr,1);
@@ -121,18 +124,10 @@ void stars_draw(cairo_t *cr)
    cairo_restore(cr);
 }
 
-int stars_ui()
+void stars_ui()
 {
-   int changes = 0;
-   if(Flags.NStars != OldFlags.NStars)
-   {
-      P("changes NStars: %d %d %d\n",changes,OldFlags.NStars,Flags.NStars);
-      OldFlags.NStars = Flags.NStars;
-      init_stars();
-      ClearScreen();
-      changes++;
-   }
-   return changes;
+   UIDO(NStars, init_stars(); ClearScreen(););
+   UIDO(Stars, ClearScreen(););
 }
 
 
@@ -144,6 +139,8 @@ int do_stars(UNUSED gpointer data)
       return TRUE;
    if (switches.UseGtk)
       return TRUE;
+   if (!Flags.Stars)
+      return TRUE;
    P("do_stars %d %d\n",NStars,counter++);
    int i;
    for (i=0; i<NStars; i++)
