@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020 Willem Vermin
+#-# 	      2019,2020,2021 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -19,14 +19,15 @@
 #-# 
 */
 #include <stdio.h>
-#include <utils.h>
 #include <string.h>
 #include <gtk/gtk.h>
 #include <stdlib.h>
 #include <X11/Intrinsic.h>
+#include "utils.h"
 #include "windows.h"
 #include "meteo.h"
 #include "debug.h"
+#include "version.h"
 
 
 Pixel Black, White;
@@ -125,9 +126,11 @@ guint add_to_mainloop(gint prio,float time,GSourceFunc func,gpointer datap)
    return g_timeout_add_full(prio,(int)1000*(time),(GSourceFunc)func,datap,NULL);
 }
 
-void remove_from_mainloop(guint tag)
+void remove_from_mainloop(guint *tag)
 {
-   g_source_remove(tag);
+   if (*tag)
+      g_source_remove(*tag);
+   *tag = 0;
 }
 
 int is_little_endian(void)
@@ -144,3 +147,16 @@ void my_cairo_paint_with_alpha(cairo_t *cr, double alpha)
       cairo_paint_with_alpha(cr,alpha);
    P("%d alpha %f\n",counter++,alpha);
 }
+
+void PrintVersion()
+{
+   printf("Xsnow-%s\n%s\n"
+	 , VERSION, VERSIONBY);
+}
+
+void rgba2color(GdkRGBA *c, char **s)
+{
+   *s = (char *)malloc(8);
+   sprintf(*s,"#%02lx%02lx%02lx",lrint(c->red*255),lrint(c->green*255),lrint(c->blue*255));
+}
+
