@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020 Willem Vermin
+#-# 	      2019,2020,2021 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 
 #include <stdio.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkx.h>
 #include <X11/Intrinsic.h>
 #include <ctype.h>
 #include "debug.h"
@@ -34,6 +35,8 @@
 #include "dsimple.h"
 #include "varia.h"
 
+#include "vroot.h"
+
 static int    do_wupdate(gpointer data);
 static void   UpdateFallenSnowRegions(void);
 static Window XWinInfo(char **name);
@@ -44,7 +47,7 @@ static int          NWindows;
 char        *SnowWinName = NULL;
 int          SnowWinX; 
 int          SnowWinY; 
-Window       RootWindow;
+Window       Rootwindow;
 unsigned int Wroot;
 unsigned int Hroot;
 int          Xroot;
@@ -57,10 +60,8 @@ Window       SnowWinb = 0;
 
 struct _switches switches;
 
-int windows_ui()
+void windows_ui()
 {
-   int changes = 0;
-   return changes;
 }
 
 void windows_draw(UNUSED cairo_t *cr)
@@ -71,7 +72,7 @@ void windows_draw(UNUSED cairo_t *cr)
 void DestroyWindow(Window w)
 {
    return;
-   if (w && w != RootWindow)
+   if (w && w != Rootwindow)
       XDestroyWindow(display,w);
 }
 
@@ -109,6 +110,19 @@ int do_wupdate(UNUSED gpointer data)
 
    if(Windows) free(Windows);
 
+   // special hack too keep SnowWin below (needed for example in FVWM/xcompmgr, 
+   // where SnowWin is not click-through)
+   {
+      P("keep below %#lx\n",SnowWin);
+      if(Flags.BelowAll)
+      {
+	 XWindowChanges changes;
+	 changes.stack_mode = Below;
+	 XConfigureWindow(display,SnowWin,CWStackMode,&changes);
+      }
+   }
+
+
    if (GetWindows(&Windows, &NWindows)<0)
    {
       I("Cannot get windows\n");
@@ -141,7 +155,7 @@ int do_wupdate(UNUSED gpointer data)
    }
 
    P("do_wupdate: %p %p\n",(void *)TransA,(void *)winfo);
-   if (SnowWin != RootWindow)
+   if (SnowWin != Rootwindow)
       if (!TransA && !winfo)
       {
 	 I("No transparent window & no SnowWin %#lx found\n",SnowWin); 
@@ -191,7 +205,8 @@ void UpdateFallenSnowRegions()
    }
    // remove fallensnow regions
    f = FsnowFirst; int nf = 0; while(f) { nf++; f = f->next; }
-   long int *toremove = (long int *)malloc(sizeof(*toremove)*nf);
+   // nf+1: prevent allocation of zero bytes
+   long int *toremove = (long int *)malloc(sizeof(*toremove)*(nf+1));
    int ntoremove = 0;
    f = FsnowFirst;
    //Atom wmState  = XInternAtom(display, "_NET_WM_STATE", True);
@@ -301,15 +316,16 @@ void UpdateFallenSnowRegions()
 //  Spin-offs:
 //     sets IsCompiz:   is this a compiz system?
 //     sets CWorkSpace: current workspace
+//     Rootwindow
 //
 //  Spin-ins:
 //     Flags
 //     display
-//     RootWindow
 //     and maybe more ...
 
 int DetermineWindow(Window *xwin, char **xwinname, GtkWidget **gtkwin, const char *transname, int *IsDesktop)
 {
+   Rootwindow = DefaultRootWindow(display);
    P("DetermineWindow\n");
    *IsDesktop = 1;
    // User supplies window id:
@@ -333,6 +349,42 @@ int DetermineWindow(Window *xwin, char **xwinname, GtkWidget **gtkwin, const cha
       *IsDesktop = 0;
       *gtkwin = NULL;
    }
+   // maybe we are started by xscreensaver: the window is in $XSCREENSAVER_WINDOW:
+   //else if (getenv("XSCREENSAVER_WINDOW"))
+   else if(Flags.ForceRoot)
+   {
+      //*xwin      = Window_With_Name(display,Rootwindow,"screensaver");
+      //*xwin = strtol(getenv("XSCREENSAVER_WINDOW"),NULL,0);
+      *xwin = DefaultRootWindow(display);
+      if (getenv("XSCREENSAVER_WINDOW"))
+      {
+	 *xwin = strtol(getenv("XSCREENSAVER_WINDOW"),NULL,0);
+	 Rootwindow = *xwin;
+      }
+      *IsDesktop = 0;
+      *gtkwin    = NULL;
+      int x,y; unsigned int w,h,b,depth;
+      Window root;
+      XGetGeometry(display,*xwin,&root,
+	    &x, &y, &w, &h, &b, &depth);
+      P("geom: %d %d %d %d\n",x,y,w,h);
+      printf("Force snow on root: window: %#lx, depth: %d\n",*xwin,depth);
+      if(0) // Trying to couple the virtual root window to gtk/cairo. No success ...
+      {
+	 GdkWindow *gdkwin;
+	 GdkDisplay *gdkdisplay;
+	 gdkdisplay = gdk_display_get_default();
+	 P("display: %p gdkdisplay: %p\n",(void*)display,(void*)gdkdisplay);
+	 gdkwin = gdk_x11_window_foreign_new_for_display(gdkdisplay,*xwin);
+	 *gtkwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	 //gtk_window_set_screen(GTK_WINDOW(*gtkwin),gdk_window_get_screen(gdkwin));
+	 gtk_widget_set_window(*gtkwin,gdkwin);
+	 P("ww: %p gdkwin: %p\n",(void*)ww,(void*)gdkwin);
+	 gtk_widget_show_all(*gtkwin);
+	 gdk_window_show(gdkwin);
+	 *IsDesktop = 1;
+      }
+   }
    else
    {
       // default behaviour
@@ -341,7 +393,7 @@ int DetermineWindow(Window *xwin, char **xwinname, GtkWidget **gtkwin, const cha
       int x,y;
       unsigned int w,h,b,depth;
       Window root;
-      XGetGeometry(display,RootWindow,&root,
+      XGetGeometry(display,Rootwindow,&root,
 	    &x, &y, &w, &h, &b, &depth);
 
       if (*gtkwin)
@@ -427,7 +479,7 @@ int DetermineWindow(Window *xwin, char **xwinname, GtkWidget **gtkwin, const cha
 	 else
 	 {
 	    printf("Using root window\n");
-	    *xwin = RootWindow;
+	    *xwin = Rootwindow;
 	 }
 	 printf("You may have to tweak 'Advanced snow settings' in the 'settings' panel.\n");
       }
@@ -468,13 +520,13 @@ void InitDisplayDimensions()
    unsigned int wroot,hroot,broot,droot;
    int xroot,yroot;
    Window root;
-   XGetGeometry(display,RootWindow,&root,
+   XGetGeometry(display,Rootwindow,&root,
 	 &xroot, &yroot, &wroot, &hroot, &broot, &droot);
    Xroot = xroot;
    Yroot = yroot;
    Wroot = wroot;
    Hroot = hroot;
-   P("InitDisplayDimensions: %d %d %d %d %d %d\n",xroot,yroot,wroot,hroot,broot,droot);
+   P("InitDisplayDimensions: %p %d %d %d %d %d %d\n",(void*)Rootwindow,xroot,yroot,wroot,hroot,broot,droot);
    DisplayDimensions();
 }
 
@@ -485,7 +537,7 @@ void DisplayDimensions()
    Window root,child_return;
 
    XGetGeometry(display,SnowWin,&root, &x, &y, &w, &h, &b, &d);
-   XTranslateCoordinates(display, SnowWin, RootWindow, 0, 0, &xr, &yr, &child_return);
+   XTranslateCoordinates(display, SnowWin, Rootwindow, 0, 0, &xr, &yr, &child_return);
    P("DisplayDimensions: %#lx %d %d %d %d %d %d %d %d\n",SnowWin,x,y,xr,yr,w,h,b,d);
    SnowWinX           = xr;// - x;
    SnowWinY           = yr;// - y;
