@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020 Willem Vermin
+#-# 	      2019,2020,2021 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -38,19 +38,20 @@
 #include "dsimple.h"
 #include "debug.h"
 
+#include "vroot.h"
+
 static void FindWindows(Display *display,Window window, long unsigned int *nwindows,Window **windows);
 static void FindWindows_r(Display *display,Window window,long unsigned int *nwindows,Window **windows);
 
+/* this one is not needed any more, but I keep the source */
 void FindWindows(Display *display,Window window,long unsigned int *nwindows,Window **windows)
 {
    *nwindows = 0;
    *windows  = NULL;
    FindWindows_r(display,window,nwindows,windows);
-   /*
    int i;
    for (i=0; i<(int)(*nwindows); i++)
-      printf("window: %#lx\n",(*windows)[i]);
-      */
+      P("window: %#lx\n",(*windows)[i]);
 }
 void FindWindows_r(Display *display,Window window,long unsigned int *nwindows,Window **windows)
 {
@@ -170,34 +171,73 @@ int GetWindows(WinInfo **windows, int *nwin)
    if(type != XA_WINDOW)
    {
       P("No _WIN_CLIENT_LIST, trying XQueryTree\n");
-      FindWindows(display,RootWindow(display,DefaultScreen(display)),&nitems,(Window **)&properties);
+      if(0)FindWindows(display,RootWindow(display,DefaultScreen(display)),&nitems,(Window **)&properties);
+      Window dummy;
+      Window *children;
+      unsigned int nchildren;
+      XQueryTree(display,DefaultRootWindow(display),&dummy,&dummy,&children,&nchildren);
+      nitems = nchildren;
+      properties = (unsigned char *)children;
    }
-   //printf("wmctrl: %d: %ld\n",__LINE__,nitems);
+   P("wmctrl: %ld\n",nitems);
    (*nwin) = nitems;
    r = (long*)properties;
-   (*windows) = (WinInfo *)malloc(nitems*sizeof(WinInfo));
+   (*windows) = NULL;
+   if(nitems>0)
+      (*windows) = (WinInfo *)malloc(nitems*sizeof(WinInfo));
    int i;
    WinInfo *w = (*windows);
-   static Atom net_atom = 0, gtk_atom = 0;
-   if(gtk_atom == 0) gtk_atom = XInternAtom(display, "_GTK_FRAME_EXTENTS", True);
-   if(net_atom == 0) net_atom = XInternAtom(display, "_NET_FRAME_EXTENTS", True);
+   static Atom net_atom = 0, gtk_atom = 0, show_atom = 0;
+   if(gtk_atom == 0)  gtk_atom  = XInternAtom(display, "_GTK_FRAME_EXTENTS",   True);
+   if(net_atom == 0)  net_atom  = XInternAtom(display, "_NET_FRAME_EXTENTS",   True);
+   if(show_atom == 0) show_atom = XInternAtom(display, "_NET_SHOWING_DESKTOP", True);
    int k = 0;
+
+   // and yet another check if window is hidden (needed e.g. in KDE/plasma after 'hide all windows')
+   int globalhidden = 0;
+   {
+      P("hidden3 %d %#lx\n",counter++,w->id);
+      if (show_atom)
+      {
+	 Atom type;
+	 unsigned long nitems, b; int format;
+	 unsigned char *properties = NULL;
+	 P("hidden3 try _NET_SHOWING_DESKTOP\n");
+	 XGetWindowProperty(display, Rootwindow, show_atom, 0, (~0L), False, 
+	       AnyPropertyType, &type, &format, &nitems, &b, &properties);
+	 if(format == 32 && nitems >=1)
+	 {
+	    if(*(long*) properties == 1)
+	       globalhidden = 1;
+	    P("hidden3 hidden:%d\n",globalhidden);
+	 }
+	 if(properties) XFree(properties);
+      }
+   }
+
    for (i=0; (unsigned long)i<nitems; i++)
    {
-      Window root,child_return;
       int x0,y0,xr,yr;
-      unsigned int bw,depth;
-      char *name;
-      XFetchName(display, r[i], &name);
-      if (name)
-	 XFree(name);
-      else
-	 continue;
+      unsigned int depth;
 
       w->id = r[i];
-      XGetGeometry (display, w->id, &root, &x0, &y0,
-	    &(w->w), &(w->h), &bw, &depth);
-      XTranslateCoordinates(display, w->id, RootWindow, 0, 0, &xr,     &yr,     &child_return);
+
+      XWindowAttributes winattr;
+      XGetWindowAttributes(display, w->id, &winattr);
+
+      x0    = winattr.x;
+      y0    = winattr.y;
+      w->w  = winattr.width;
+      w->h  = winattr.height;
+      depth = winattr.depth;
+
+      P("%d %#lx %d %d %d %d %d\n",counter++,w->id,x0,y0,w->w,w->h,depth);
+      // if this window is showing nothing, we ignore it:
+      if (depth == 0)
+	 continue;
+
+      Window child_return;
+      XTranslateCoordinates(display, w->id, Rootwindow, 0, 0, &xr,     &yr,     &child_return);
       w->xa = xr - x0;
       w->ya = yr - y0;
 
@@ -280,31 +320,63 @@ int GetWindows(WinInfo **windows, int *nwin)
       if(properties) XFree(properties);
 
       // check if window is hidden
-      w->hidden = 0;
-      properties = NULL;
-      nitems = 0;
-      atom  = XInternAtom(display, "_NET_WM_STATE", True);
-      XGetWindowProperty(display, w->id, atom, 0, (~0L), False, 
-	    AnyPropertyType, &type, &format, &nitems, &b, &properties);
-      if(format == 32)
+      w->hidden = globalhidden;
+      if(!w->hidden)
       {
-	 unsigned long i;
-	 for (i=0; i<nitems; i++)
+	 if (winattr.map_state != IsViewable)
 	 {
-	    char *s = NULL;
-	    s = XGetAtomName(display,((Atom*)properties)[i]);
-	    if (!strcmp(s,"_NET_WM_STATE_HIDDEN"))
-	    { 
-	       P("%#lx is hidden %d\n",f->id, counter++);
-	       w->hidden = 1;
-	       if(s) XFree(s);
-	       break;
-	    }
-	    if(s) XFree(s);
+	    P("map_state: %#lx %d\n",w->id,winattr.map_state);
+	    w->hidden = 1;
 	 }
       }
-      if(properties) XFree(properties);
+      // another check on hidden
+      if (!w->hidden)
+      {
+	 properties = NULL;
+	 nitems = 0;
+	 atom  = XInternAtom(display, "_NET_WM_STATE", True);
+	 XGetWindowProperty(display, w->id, atom, 0, (~0L), False, 
+	       AnyPropertyType, &type, &format, &nitems, &b, &properties);
+	 if(format == 32)
+	 {
+	    unsigned long i;
+	    for (i=0; i<nitems; i++)
+	    {
+	       char *s = NULL;
+	       s = XGetAtomName(display,((Atom*)properties)[i]);
+	       if (!strcmp(s,"_NET_WM_STATE_HIDDEN"))
+	       { 
+		  P("%#lx is hidden %d\n",f->id, counter++);
+		  w->hidden = 1;
+		  if(s) XFree(s);
+		  break;
+	       }
+	       if(s) XFree(s);
+	    }
+	 }
+	 if(properties) XFree(properties);
+      }
 
+      // yet another check if window is hidden:
+      if(!w->hidden)
+      {
+	 P("hidden2 %#lx\n",w->id);
+	 properties = NULL;
+	 nitems = 0;
+	 atom  = XInternAtom(display, "WM_STATE", True);
+	 XGetWindowProperty(display, w->id, atom, 0, (~0L), False, 
+	       AnyPropertyType, &type, &format, &nitems, &b, &properties);
+	 if(format == 32 && nitems >=1)
+	 {
+	    // see https://tronche.com/gui/x/icccm/sec-4.html#s-4.1.3.1
+	    // WithDrawnState: 0
+	    // NormalState:    1
+	    // IconicState:    3
+	    if(*(long*) properties != NormalState)
+	       w->hidden = 1;
+	 }
+	 if(properties) XFree(properties);
+      }
 
       properties = NULL;
       nitems = 0;
@@ -358,7 +430,10 @@ int GetWindows(WinInfo **windows, int *nwin)
       {
 	 // this is a problem....
 	 // In for example TWM, neither NET nor GTK is the case.
-	 // But, there is some window decoration, in 
+	 // Let us try this one:
+	 w->x = x0;
+	 w->y = y0;
+	 P("%d %#lx %d %d\n",counter++,w->id,w->x,w->y);
       }
       if(properties)XFree(properties);
       w++;
