@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020 Willem Vermin
+#-# 	      2019,2020,2021 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -23,81 +23,12 @@
 #include <X11/Xutil.h>
 
 
-#define DEFAULT_AllWorkspaces 1
-#define DEFAULT_BelowAll 1
-#define DEFAULT_BGColor "#000000"
-#define DEFAULT_BlowOffFactor 40
-#define DEFAULT_CpuLoad 100
-#define DEFAULT_Transparency 0
-#define DEFAULT_Defaults 0
-#define DEFAULT_DesiredNumberOfTrees 10
-#define DEFAULT_Desktop 0
-#define DEFAULT_DisplayName ""
-#define DEFAULT_Done 0
-#define DEFAULT_Exposures -SOMENUMBER
-#define DEFAULT_FlakeCountMax 300 
-#define DEFAULT_FullScreen 0
-#define DEFAULT_MaxOnTrees 200
-#define DEFAULT_MaxScrSnowDepth 50    /* Will build up to MAXSRCSNOWDEPTH */
-#define DEFAULT_MaxWinSnowDepth 30  // wwvv
-#define DEFAULT_NoBlowSnow 0
-#define DEFAULT_NoConfig 0
-#define DEFAULT_NoFluffy 0
-#define DEFAULT_NoKeepSBot 0
-#define DEFAULT_NoKeepSnow 0
-#define DEFAULT_NoKeepSnowOnTrees 0
-#define DEFAULT_NoKeepSWin 0
-#define DEFAULT_NoMenu 0
-#define DEFAULT_NoMeteorites 0
-#define DEFAULT_NoRudolf 0 
-#define DEFAULT_NoSanta 0
-#define DEFAULT_NoSnowFlakes 0
-#define DEFAULT_NoTrees 0
-#define DEFAULT_NoWind 0
-#define DEFAULT_NStars 20
-#define DEFAULT_OffsetS 0 
-#define DEFAULT_OffsetW -8
-#define DEFAULT_OffsetX 4
-#define DEFAULT_OffsetY 0 
-#define DEFAULT_Quiet 1
-#define DEFAULT_SantaSize 3         // default santa size    
-#define DEFAULT_SantaSpeedFactor 100
-#define DEFAULT_SnowColor "snow"
-#define DEFAULT_SnowFlakesFactor 100
-#define DEFAULT_SnowSize 8 
-#define DEFAULT_SnowSpeedFactor 100
-#define DEFAULT_StopAfter -1
-#define DEFAULT_TreeColor "chartreuse" // The author thoroughly recommends a cup of tea with a dash of green Chartreuse. Jum!         */
-#define DEFAULT_TreeFill 30
-#define DEFAULT_TreeType "1,2,3,4,5,6,7,"       // default treetype
-#define DEFAULT_UseAlpha  SOMENUMBER
-#define DEFAULT_UseBG 0 
-#define DEFAULT_WhirlFactor 100
-#define DEFAULT_WindNow 0
-#define DEFAULT_WindowId 0
-#define DEFAULT_WindTimer 30
-#define DEFAULT_XWinInfoHandling 0
-#define DEFAULT_WantWindow 0
-#define DEFAULTTREETYPE 2
-#define DEFAULT_Noisy 0
-#define ALLTREETYPES "0" DEFAULT_TreeType
-
-
-#define VINTAGE_DesiredNumberOfTrees 6
-#define VINTAGE_NoBlowSnow 1
-#define VINTAGE_NoKeepSnowOnTrees 1
-#define VINTAGE_NoMeteorites 1
-#define VINTAGE_NoRudolf 0 
-#define VINTAGE_NStars 0
-#define VINTAGE_SantaSize 2 
-#define VINTAGE_SnowFlakesFactor 15
-#define VINTAGE_TreeType "0,"
 
 #define FLAGSFILE ".xsnowrc"
 #define FLAKES_PER_SEC_PER_PIXEL 30
 #define INITIALSCRPAINTSNOWDEPTH 8  /* Painted in advance */
 #define INITIALYSPEED 120   // has to do with vertical flake speed
-#define MAXBLOWOFFFACTOR 10
+#define MAXBLOWOFFFACTOR 100
 #define MAXSANTA	4    // santa types 0..4
 #define MAXTANNENPLACES 10   // number of trees
 #define MAXTREETYPE 7        // treetypes: 0..MAXTREETYPE
@@ -116,29 +47,6 @@
 #define SNOWSPEED 0.7    // the higher, the speedier the snow
 #define WHIRL 150
 
-/* birds */
-#define DEFAULT_Anarchy 50    /*percent*/
-#define DEFAULT_Neighbours 7
-//#define DEFAULT_Range 20      /*m*/
-#define DEFAULT_Nbirds 70
-#define DEFAULT_ShowBirds 1
-#define DEFAULT_BirdsOnly 0
-#define DEFAULT_PrefDistance 20
-#define DEFAULT_BirdsRestart 0
-//#define DEFAULT_ViewingDistance PreferredViewingDistance()
-#define DEFAULT_ViewingDistance 40
-#define DEFAULT_BirdsSpeed 100
-#define DEFAULT_AttrFactor 100
-#define DEFAULT_DisWeight 20
-#define DEFAULT_FollowWeight 30
-#define DEFAULT_FollowSanta 1
-#define DEFAULT_BirdsColor "#361A07"
-#define DEFAULT_ShowAttrPoint 0
-#define DEFAULT_BirdsScale 100
-
-#define VINTAGE_ShowBirds 0
-#define VINTAGE_BirdsOnly 0
-#define NBIRDS_MAX 1000
 
 // timers
 
@@ -157,8 +65,9 @@
 #define time_sfallen          2.30             /* time between smoothing of fallen snow      */
 #define time_snow_on_trees    0.50             /* time between redrawings of snow on trees   */
 #define time_star             0.50             /* time between drawing stars                 */ 
-#define time_testing          1.10             /* time between testing code                  */ 
+#define time_testing          2.10             /* time between testing code                  */ 
 #define time_ui_check         0.25             /* time between checking values from ui       */ 
+#define time_umoon            0.02            /* time between update position of moon       */
 #define time_usanta           0.02             /* time between update of santa position      */
 #define time_ustar            2.00             /* time between updating stars                */ 
 #define time_wind             0.10             /* time between starting or ending wind       */
